*
*  X3CNVCHR.PRG
*  Converts the passed character string to a value
*  of the passed data type.  Returns .NULL. if 
*  unable to process.
*
*  Copyright (c) 1996  (Public Domain) MaxTech, Inc. 
*  Copyright (c) 1996  (Public Domain) Drew Speedie
*                      MaxTech, Inc.
*                      8721C Plantation Lane, Suite 301
*                      Manassas, VA  22110
*                      703-369-1759
*  Author:  Drew Speedie
*
*  lParameters
*      tcString (R) Character string to be converted
*    tcDataType (R) Data type to convert the string to:
*                     "C"haracter
*                     "N"umeric
*                     "D"ate
*                     Date"T"ime
*                     "L"ogical
*                     "E"xpression
*
lparameters tcString, tcDataType
local luReturnValue, lcString, lcDataType
luReturnValue = .NULL.
IF type("tcString") = "C"
  lcString = tcString
 ELSE
  return luReturnValue
ENDIF
IF type("tcDataType") = "C" AND !empty(tcDataType)
  lcDataType = upper(alltrim(left(tcDataType,1)))
 ELSE
  return luReturnValue
ENDIF 
IF !inlist(lcDataType,"C","N","D","L","T","E")
  return luReturnValue
ENDIF
DO CASE
  **********************************************
  CASE lcDataType = "C"   &&& character
  **********************************************
    luReturnValue = alltrim(lcString)
  **********************************************
  CASE lcDataType = "N"   &&& numeric
  **********************************************
    IF empty(lcString)
      luReturnValue = 0
     ELSE
      lcString = alltrim(lcString)
      local lnDecimals
      lnDecimals = 0
      IF "." $ lcString
        lnDecimals = substr(lcString,at(".",lcString)+1)
        lnDecimals = len(lnDecimals)
      ENDIF
      IF lnDecimals > 0
        local lnSetDecimals
        lnSetDecimals = set("DECIMALS")
        set decimals to lnDecimals
        luReturnValue = val(lcString)
        set decimals to lnSetDecimals
       ELSE
        luReturnValue = int(val(lcString))
      ENDIF   
    ENDIF empty(lcString)
  **********************************************
  CASE lcDataType = "D"   &&& date
  **********************************************
    IF empty(lcString) ;
         OR empty(chrtran(lcString,"/",space(0)))
      luReturnValue = {}
     ELSE
      luReturnValue = ctod(alltrim(lcString))
    ENDIF empty(lcString)
  **********************************************
  CASE lcDataType = "T"   &&& datetime
  **********************************************
    lcString = alltrim(lcString)
    IF empty(lcString)
      luReturnValue = ctot("{}")
     ELSE
      luReturnValue = ctot(lcString)
    ENDIF empty(lcString)
  **********************************************
  CASE lcDataType = "L"   &&& logical
  **********************************************
    lcString = upper(alltrim(lcString))
    DO CASE
      CASE empty(lcString)
        * unable to process
      CASE inlist(lcString,"T",".T.","TRUE","Y","YES")
        luReturnValue = .t.
      CASE inlist(lcString,"F",".F.","FALSE","N","NO")
        luReturnValue = .f.
      OTHERWISE
        * unable to process
    ENDCASE  
  **********************************************
  CASE lcDataType = "E"   &&& expression
  **********************************************
    IF empty(lcString)
     ELSE
      local lcOnError, llError
      lcOnError = on("ERROR")
      llError = .f.
      on error llError = .t.
      luReturnValue = eval(alltrim(lcString))
      IF llError
        luReturnValue = .NULL.
      ENDIF
      on error &lcOnError  
    ENDIF empty(lcString)
ENDCASE
return luReturnValue  