*
*  X3FWDATA.PRG
*  Create the XX database/xxSYSTEM, xxUSERS, xxSYSID, xxREPCAT
*  tables in the current directory
*
*  Copyright (c) 1996  (Public Domain) MaxTech, Inc.
*  Copyright (c) 1996  (Public Domain) Drew Speedie
*                      MaxTech, Inc.
*                      8721C Plantation Lane, Suite 302
*                      Manassas, VA  22110
*                      703-369-1759
*  Author:  Drew Speedie
*
*
*  Version 1.3
*
*
*  Usage:
*  do x3fwdata with "MT","ALL"          &&& create MT database, MTSYSTEM.DBF, MTGENPK.DBF, MTUSERS.DBF, MTREPCAT.DBF
*
*  Usage:
*  do x3fwdata with "MT","SYSTEM"       &&& create MT database, MTSYSTEM.DBF, MTLIBCLS.DBF in MT.DBC
*  do x3fwdata with "MT","GENPK"        &&& create MTGENPK.DBF in MT.DBC
*  do x3fwdata with "MT","USERS"        &&& create MTUSERS.DBF in MT.DBC
*  do x3fwdata with "MT","REPCAT"       &&& create MTREPCAT.DBF in MT.DBC
*
lparameters tcParameter1, tcParameter2, tlParameter3
local lcAction, ;
      lcXLibraryDirectory, ;
      lcSystemTable, ;
      llOverwriteExistingX3DBC, ;
      llDontSetDatabaseTo, ;
      lcCurrentSetDatabaseTo
IF type("tcParameter1")#"C" OR empty(tcParameter1)
  wait window "Parameters passed incorrectly to " + program()
  return .f.
ENDIF
lcAction = upper(alltrim(tcParameter1))
lcCurrentSetDatabaseTo = set("DATABASE")
IF len(lcAction) # 2
  wait window "Invalid application prefix parameter passed to " + program()
  return .f.
ENDIF
IF type("tcParameter2")#"C" OR empty(tcParameter2)
  wait window "Invalid system table name parameter passed to " + program()
  return .f.
ENDIF
lcSystemTable = upper(alltrim(tcParameter2))
IF !inlist(lcSystemTable,"SYSTEM","GENPK","USERS","REPCAT","ALL")
  wait window "Invalid system table name parameter passed to " + program()
  return .f.
ENDIF
*
*  pass a .T. as the 3rd parameter to leave
*  the current SET DATABASE TO instead of
*  arbitrarily changing it to the 2-character
*  prefix
*
IF type("tlParameter3")#"L"
  llDontSetDatabaseTo = .f.
 ELSE
  llDontSetDatabaseTo = tlParameter3
ENDIF
IF llDontSetDatabaseTo
  lcCurrentSetDatabaseTo = set("DATABASE")
ENDIF
release all except l*
clear windows
clear popups
clear menus
clear dlls
close all
IF llDontSetDatabaseTo
  open database (lcCurrentSetDatabaseTo)
  set database to (lcCurrentSetDatabaseTo)
ENDIF
set talk off
set safety off
set exclusive on
IF lcSystemTable = "ALL"
  do CreateXXData with lcAction, "SYSTEM", llDontSetDataBaseTo
  do CreateXXData with lcAction, "GENPK",  llDontSetDataBaseTo
  do CreateXXData with lcAction, "USERS",  llDontSetDataBaseTo
  do CreateXXData with lcAction, "REPCAT", llDontSetDataBaseTo
 ELSE
  do CreateXXData with lcAction, lcSystemTable, llDontSetDataBaseTo
ENDIF
return


PROCEDURE CreateXXData
lparameters tcAppPrefix, tcTableName, tlDontSetDatabaseTo
wait window "Creating " + tcAppPrefix + " system tables..." nowait
IF tcTableName = "SYSTEM"
  IF file(tcAppPrefix+".DBC")
    *
    * if we're creating the SYSTEM tables, we need to start over
    *
    delete database (tcAppPrefix+".DBC") deletetables
  ENDIF
  create database (tcAppPrefix)
ENDIF
IF tlDontSetDatabaseTo
 ELSE
  open database (tcAppPrefix)
  set database to (tcAppPrefix)
ENDIF

DO CASE

  CASE tcTableName = "SYSTEM"
    do CreateSystemTables with tcAppPrefix
  CASE tcTableName = "GENPK"
    do CreateGeneratedPKTable with tcAppPrefix
  CASE tcTableName = "USERS"
    do CreateUsersTable with tcAppPrefix
  CASE tcTableName = "REPCAT"
        do CreatereportTable with tcAppPrefix
ENDCASE
return



PROCEDURE CreateSystemTables
lparameters tcAppPrefix
*
* system data
*
create table (tcAppPrefix+"SYSTEM") name APPINFO ;
     (App_ItemPK          C(20), ;
      App_ItemDataType    C(1), ;
      App_ItemValue       C(100),;
      App_ItemDescription M(4))
select APPINFO
wait window alias() nowait
alter table APPINFO add primary key upper(App_ItemPK) tag App_ItemPK
=dbsetprop("APPINFO",                     "TABLE", "Comment", "Application-level information")
=dbsetprop("APPINFO.App_ItemPK",          "FIELD", "Comment", "Application setup/configuration item (Primary Key)")
=dbsetprop("APPINFO.App_ItemDataType",    "FIELD", "Comment", "Data type for App_Item (C,N,L,D,T)")
=dbsetprop("APPINFO.App_ItemValue",       "FIELD", "Comment", "App_Item value")
=dbsetprop("APPINFO.App_ItemDescription", "FIELD", "Comment", "Description of App_Item")

m.App_ItemPK          = "DevModeExpr"
m.App_ItemDataType    = "E"
m.App_ItemValue       = [getenv("FPDEV")="ON"]
m.App_ItemDescription = "Expression indicating whether or not app is running in development"
insert into APPINFO from memvar
m.App_ItemPK          = "SetProcedure"
m.App_ItemDataType    = "C"
m.App_ItemValue       = space(0)
m.App_ItemDescription = "List of procedure files to SET PROCEDURE TO"
insert into APPINFO from memvar
m.App_ItemPK          = "SetClasslib"
m.App_ItemDataType    = "C"
m.App_ItemValue     = space(0)
m.App_ItemDescription = "List of class library .VCX files to SET CLASSLIB TO"
insert into APPINFO from memvar
m.App_ItemPK          = "SetLibrary"
m.App_ItemDataType    = "C"
m.App_ItemValue       = space(0)
m.App_ItemDescription = "List of API library files to SET LIBRARY TO"
insert into APPINFO from memvar
m.App_ItemPK          = "SetPathDev"
m.App_ItemDataType    = "C"
m.App_ItemValue       = space(0)
m.App_ItemDescription = "VFP path to be set in application setup -- Development"
insert into APPINFO from memvar
m.App_ItemPK          = "SetPathProd"
m.App_ItemDataType    = "C"
m.App_ItemValue       = space(0)
m.App_ItemDescription = "VFP path to be set in application setup -- Production"
insert into APPINFO from memvar
m.App_ItemPK          = "Title"
m.App_ItemDataType    = "C"
m.App_ItemValue       = tcAppPrefix + " Application"
m.App_ItemDescription = "Title string for use in messaging"
insert into APPINFO from memvar
m.App_ItemPK          = "IconFile"
m.App_ItemDataType    = "C"
m.App_ItemValue       = space(0)
m.App_ItemDescription = ".ICO file for the application"
insert into APPINFO from memvar
m.App_ItemPK          = "ScreenWallpaperFile"
m.App_ItemDataType    = "C"
m.App_ItemValue       = space(0)
m.App_ItemDescription = ".BMP file for the use as screen wallpaper"
insert into APPINFO from memvar
m.App_ItemPK          = "UserLoginFormClass"
m.App_ItemDataType    = "C"
m.App_ItemValue       = "frmLogin,X3FW.VCX"
m.App_ItemDescription = "Optional way to set User Login form class/.VCX"
insert into APPINFO from memvar
m.App_ItemPK          = "DevToolbarClass"
m.App_ItemDataType    = "C"
IF file("X3TOOLS.VCX")
  m.App_ItemValue     = "tbrDevTools,X3TOOLS.VCX"
 ELSE
  m.App_ItemValue     = space(0)
ENDIF
m.App_ItemDescription = "Class name for any desired Developers Toolbar"
insert into APPINFO from memvar
m.App_ItemPK          = "DevToolbarPad"
m.App_ItemDataType    = "C"
m.App_ItemValue       = "System"
m.App_ItemDescription = "Pad prompt of system menu popup to which to add Dev toolbar"
insert into APPINFO from memvar
m.App_ItemPK          = "LoginControl"
m.App_ItemDataType    = "C"
m.App_ItemValue       = "AUTO"
m.App_ItemDescription = "Type of auto-login control desired"
insert into APPINFO from memvar
m.App_ItemPK          = "AutoLoginUserID"
m.App_ItemDataType    = "C"
m.App_ItemValue       = "DEFAULT"
m.App_ItemDescription = "User ID to use for auto-login"
insert into APPINFO from memvar
m.App_ItemPK          = "AutoLoginPassword"
m.App_ItemDataType    = "C"
m.App_ItemValue       = "PASSWORD"
m.App_ItemDescription = "Password to use for auto-login"
insert into APPINFO from memvar
m.App_ItemPK          = "AppTimerInterval"
m.App_ItemDataType    = "N"
m.App_ItemValue       = space(0)
m.App_ItemDescription = "Interval for the application timer"
insert into APPINFO from memvar
m.App_ItemPK          = "TimeoutSeconds"
m.App_ItemDataType    = "N"
m.App_ItemValue       = space(0)
m.App_ItemDescription = "Number of seconds before timeout"
insert into APPINFO from memvar
m.App_ItemPK          = "TimeoutType"
m.App_ItemDataType    = "C"
m.App_ItemValue       = "N"
m.App_ItemDescription = [Timeout type "A"pp or "M"enu or "N"one]
insert into APPINFO from memvar
return



PROCEDURE CreateGeneratedPKTable
*
* system-generated PK data
*
lparameters tcAP
create table (tcAP+"GENPK") name "GENERATEPK" ;
     (GPK_PK            C(40), ;
      GPK_CurrentNumber I(4), ;
      GPK_CurrentAlpha  C(5))
alter table GENERATEPK add primary key GPK_PK tag GPK_PK
index on deleted() tag GPK_Del
=dbsetprop("GENERATEPK",                   "TABLE","Comment","System-generated Primary Keys")
=dbsetprop("GENERATEPK.GPK_PK",            "FIELD","Caption","Primary Key")
=dbsetprop("GENERATEPK.GPK_PK",            "FIELD","Comment","TableName for which to generate PKs")
=dbsetprop("GENERATEPK.GPK_CurrentNumber", "FIELD","Caption","Current PK")
=dbsetprop("GENERATEPK.GPK_CurrentNumber", "FIELD","Comment","Current PK Number in use")
=dbsetprop("GENERATEPK.GPK_CurrentAlpha",  "FIELD","Caption","Current AlphaNumeric PK")
=dbsetprop("GENERATEPK.GPK_CurrentAlpha",  "FIELD","Comment","Current AlphaNumeric PK in use")
return



PROCEDURE CreateUsersTable
lparameters tcAppPrefix
create table (tcAppPrefix+"USERS") name USERS ;
     (Usr_PK                 C(5), ;
      Usr_LanID              C(8), ;
      Usr_Password           C(8) default "PASSWORD", ;
      Usr_FirstName          C(15), ;
      Usr_MiddleInitial      C(1), ;
      Usr_LastName           C(15), ;
      Usr_Phone              C(12), ;
      Usr_Fax                C(12), ;
      Usr_Notes              M(4), ;
      Usr_ScreenWindowState  I(4), ;
      Usr_DevTbrDockPosition I(4) NULL)
alter table USERS add primary key Usr_PK tag Usr_PK
select USERS
index on upper(Usr_LastName) + upper(Usr_FirstName) + upper(Usr_MiddleInitial) tag Usr_Name
index on upper(Usr_LanID) + Usr_PassWord tag Usr_IDPass
index on Usr_LanID tag Usr_LanID candidate for !deleted()
index on deleted() tag Usr_Del
=dbsetprop("USERS",                        "TABLE","Comment","Users")
=dbsetprop("USERS.Usr_PK",                "FIELD","Caption","Primary Key")
=dbsetprop("USERS.Usr_PK",                "FIELD","Comment","System-generated Primary Key")
=dbsetprop("USERS.Usr_LanID",             "FIELD","Caption","LAN ID")
=dbsetprop("USERS.Usr_LanID",             "FIELD","Comment","User's LAN ID")
=dbsetprop("USERS.Usr_Password",          "FIELD","Caption","Password")
=dbsetprop("USERS.Usr_Password",          "FIELD","Comment","User's Password")
=dbsetprop("USERS.Usr_FirstName",         "FIELD","Caption","First Name")
=dbsetprop("USERS.Usr_MiddleInitial",     "FIELD","Caption","Middle Initial")
=dbsetprop("USERS.Usr_LastName",          "FIELD","Caption","Last Name")
=dbsetprop("USERS.Usr_Phone",             "FIELD","Caption","Phone Number")
=dbsetprop("USERS.Usr_Fax",               "FIELD","Caption","Fax Number")
=dbsetprop("USERS.Usr_Notes",             "FIELD","Caption","Notes")
=dbsetprop("USERS.Usr_ScreenWindowState", "FIELD","Caption","Screen Max on Startup")
=dbsetprop("USERS.Usr_ScreenWindowState", "FIELD","Comment","Integer Value indicating _Screen.WindowState on startup (saved each time the app is exited)")
=dbsetprop("USERS.Usr_DevTbrDockPosition","FIELD","Caption","Dev Toolbar Dock on Startup")
=dbsetprop("USERS.Usr_DevTbrDockPosition","FIELD","Comment","Integer Value indicating oApp.oDevToolbar.DockPosition on startup (saved each time the app is exited)")

select USERS
wait window alias() nowait
insert into USERS (Usr_PK, ;
                   Usr_FirstName, ;
                   Usr_MiddleInitial, ;
                   Usr_LastName, ;
                   Usr_LanID, ;
                   Usr_DevTbrDockPosition) ;
                   values ;
                  ("DEFA", ;
                   "Dudley", ;
                   "E", ;
                   "Fault", ;
                   "DEFAULT", ;
                   -1)
insert into USERS (Usr_PK, ;
                   Usr_FirstName, ;
                   Usr_MiddleInitial, ;
                   Usr_LastName, ;
                   Usr_LanID, ;
                   Usr_DevTbrDockPosition) ;
                   values ;
                  ("EXTR", ;
                   "External", ;
                   "X", ;
                   "Ternal", ;
                   "EXTERNAL", ;
                   -1)

local lcDatabasePlusTable
lcDatabasePlusTable = cursorgetprop("DATABASE","USERS") + "!USERS"
CREATE SQL VIEW LV_USERS AS ;
    SELECT * ;
    FROM &lcDatabasePlusTable ;
    WHERE USERS.Usr_pk = ?lcUsr_PK
=DBSetProp( 'LV_USERS', 'View', 'UpdateType', 1 )
=DBSetProp( 'LV_USERS', 'View', 'WhereType', 3 )
=DBSetProp( 'LV_USERS', 'View', 'FetchMemo', .T. )
=DBSetProp( 'LV_USERS', 'View', 'SendUpdates', .T. )
=DBSetProp( 'LV_USERS', 'View', 'UseMemoSize', 255 )
=DBSetProp( 'LV_USERS', 'View', 'FetchSize', 100 )
=DBSetProp( 'LV_USERS', 'View', 'MaxRecords', -1 )
=DBSetProp( 'LV_USERS', 'View', 'Tables', lcDatabasePlusTable )
=DBSetProp( 'LV_USERS.usr_pk', 'Field', 'KeyField', .T. )
=DBSetProp( 'LV_USERS.usr_pk', 'Field', 'Updatable', .T. )
=DBSetProp( 'LV_USERS.usr_pk', 'Field', 'UpdateName', lcDatabasePlusTable+".Usr_pk" )
return


PROCEDURE CreateReportTable
lparameters tcAppPrefix
create table (tcAppPrefix+"REPCAT") name REPORTCATALOG ;
                (Rep_PK   C(5),;
         Rep_File C(8), ;
         Rep_Name C(40), ;
         Rep_Description M(4))

alter table REPORTCATALOG add primary key Rep_PK tag Rep_PK
INDEX ON UPPER(Rep_NAME) TAG Rep_Name

= DBSETPROP( "REPORTCATALOG.Rep_PK",          "Field", "Caption", "Primary Key" )
= DBSETPROP( "REPORTCATALOG.Rep_File",        "Field", "Caption", ".FRX File Name" )
= DBSETPROP( "REPORTCATALOG.Rep_Name",        "Field", "Caption", "Name" )
= DBSETPROP( "REPORTCATALOG.Rep_Description", "Field", "Caption", "Description" )

return
